#pylint: disable=C0301,W0212,W0613,C0103

"""
We mostly use the PySRP library to implement SRP, but our protocol involves
making a few small changes to the SRP spec. To realize those changes, we
monkeypatch the corresponding functions in PySRP.
"""

import srp
from Crypto.Random import random

def patch():
    """Call this method after importing srp to complete the monkeypatch"""

    def get_ng( ng_type, n_hex, g_hex ):
        """Change this function to return longs, as opposed to ints"""
        if ng_type < srp.NG_CUSTOM:
            n_hex, g_hex = srp._ng_const[ ng_type ]
        return long(n_hex, 16), long(g_hex, 16)
    srp.get_ng = get_ng

    def gen_x(hash_class, salt, username, password):
        """Instead of x=H(p,s) we merely use x=p+s"""
        return password + salt
    srp.gen_x = gen_x

    def create_salted_verification_key(commitment_P, q, ng_type=srp.NG_2048, n_hex=None, g_hex=None):
        """We must modify how the server computes its verification key: vkey = commitment_P * g^s"""

        if ng_type == srp.NG_CUSTOM and (n_hex is None or g_hex is None):
            raise ValueError("Both n_hex and g_hex are required when ng_type = NG_CUSTOM")
        N, g = srp.get_ng(ng_type, n_hex, g_hex)
        _s = random.randint(0, q-1)
        _v = srp.long_to_bytes( (commitment_P * pow(g, _s, N)) % N)
        _s = srp.long_to_bytes( _s )
        return _s, _v
    srp.create_salted_verification_key = create_salted_verification_key
